# Generated by Django 4.0.dev20210817102354 on 2022-01-06 10:25

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('taggit', '0003_taggeditem_add_unique_index'),
        ('demosite', '0002_capitalizeverbose'),
    ]

    operations = [
        migrations.AlterField(
            model_name='blogentrypagetag',
            name='tag',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='%(app_label)s_%(class)s_items', to='taggit.tag'),
        ),
    ]
