/*
 * Styling for the workflow progress view
 * See: wagtail/wagtail/admin/templates/wagtailadmin/pages/workflow_history/detail.html
 */

.workflow-progress-tabs {
    &__buttons {
        margin-top: 30px;
    }

    &__button {
        background: none;
        border: 0;
        padding-bottom: 5px;
        font-weight: bold;

        &--active {
            border-bottom: 3px solid #007d7e;
        }
    }

    &__tab {
        display: none;

        &--active {
            display: block;
        }
    }
}

.workflow-progress-table {
    width: 100%;
    border-top: 1px solid rgb(229, 229, 229);
    border-bottom: 1px solid rgb(229, 229, 229);
    font-size: 0.8em;

    td,
    th {
        padding: 20px;
    }

    th {
        font-size: 1.2em;
    }

    &__left-column {
        font-size: 1.2em;
        font-weight: bold;
        color: #555;
    }

    &--tasks {
        background-color: rgb(250, 250, 250);

        .workflow-progress-table__left-column {
            background-color: rgb(245, 245, 245);
        }
    }

    &--timeline tr {
        border-bottom: 1px solid rgb(229, 229, 229);
    }
}
